/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CPIPE_H
#define CPIPE_H

#include <stdio.h>
#include <iostream>
#include <string.h>
#include <vector>

#if defined(__WIN32__) || defined(__CYGWIN__)		/* WIN32 Implementation */
#include <windows.h>
#else							/* Unix-like Implementation */
#include <sys/wait.h>
#include <assert.h>
#include <stdlib.h>
#include <unistd.h>					
#endif

#include "IStreamDaq.h"

/**
 *\file CPipe.h
 *\date  Created on: 22-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief Class to read from a pipe (tread intercommunication method). 
 *	 Implementation of IStream interface to read a stream from a pipe.
*	 This class reads bytes from a  pipe and creates packets shaped 
*	  like the SDef structure directives. Daq is client of the pipe
*	  created by the server thread
*/


/**
 *\class CPipe
 *\brief Class to read from a pipe (tread intercommunication method). 
 *	 Implementation of IStream interface to read a stream from a pipe.
*	 This class reads bytes from a  pipe and creates packets shaped 
*	  like the SDef structure directives. Daq is client of the pipe
*	  created by the server thread
*/

class CPipe: implements IStreamDaq{
	public:
		/**\brief CPipe destructor*/		
		~CPipe();

		/**\brief Get the stream definition structure from xml parser and open file
		*\param def 	Filled stream definition structure (input) 
		*\param sty		Style of reading (by length, or by start and end character)
		*\return 	s_ret value about opening the port and about the 
		*/
		virtual s_ret getDefinition (struct SDef def, s_read_style sty);
		
		/**\brief Blocking method to wait the data to be passed to formatter
		 *	  (maybe after a decode and a "divide in message" processes 
		*	  respectively to interpret the escape charactera and to divide
		*	  packets to the correct length).
		*\param buffer 	Pointer to a pointer of a vector of byte to be formatted (output)
		*\param length 	Pointer to the packet lenght variable (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char **buffer, unsigned long *length);

	private:		

		/**\brief Connection to a certain pipe like a client
		*\return 	s_ret value about connecting to certain pipe \see s_ret
		*/
		s_ret Pipe_init(void);	

		/**\brief Closing the connection to the pipe*/
		void Pipe_close(void);							

		/**\brief Receive from pipe
		*\param data 		Buffer to receive data
		*\param data_size 	Size of data to receive
		*\return 	s_ret value about receiving from \see s_ret
		*/
		s_ret Pipe_read(unsigned char* data, unsigned int data_size);	
		
		/**\brief Send by the pipe
		*\param data 		Data buffer to send
		*\param data_size 	Size of data to send
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Pipe_write(unsigned char* data, unsigned int data_size);	
		
		/**\brief This function execute the handshaking to initialize the stream
		*\param  style 	Handshacking style
		*\return 	s_ret value about sending from \see s_ret
		*/
		s_ret Pipe_handshaking(handshake_style style);

		#if defined(__WIN32__) || defined(__CYGWIN__)		/* WIN32 Implementation */
		HANDLE 					pi_Handler;	/**< Pipe handle*/
		#else							/* Unix-like Implementation */
		int 					pfd[2];		/**< Pair of file descriptors, 
									pointing to a pipe inode, and places 
									them in the array pointed to by filedes. 
									filedes[0] is for reading, filedes[1] 
									is for writing.*/
    		pid_t 					cpid;		/**< Pipe handle*/
		#endif
	
		char*					pi_Name;	/**< Pipe name */
		unsigned long				pi_Size;	/**< Packet total size */
		unsigned long				pi_Size_r;	/**< Received packet total size */
		s_read_style				pi_Style;	/**< Reading style*/
		handshake_style				pi_HStyle;	/**< Handsaking style*/
		unsigned char				pi_Start;	/**< Start character */
		unsigned char				pi_End;		/**< End character */
		
		std::vector<struct SHandshaking>*	pi_Handshk;	/**< Handshaking packets*/
	
};
#endif
