/*
Copyright (C) 2009 -  Claudio Salvadori and Paolo Pagano
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/**
 *\file LoopDAQEvent.cpp
 *\date  Created on: 26-08-2009
 *\author  Author: Claudio Salvadori and Paolo Pagano
 *\brief  Post-processing functions */

#include "TH1D.h"
#include "TGraph.h"
#include <stdlib.h>
#include "Riostream.h"
#include "TROOT.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TNetFile.h"
#include "TRandom.h"
#include "TTree.h"
#include "TBranch.h"
#include "TClonesArray.h"
#include "TStopwatch.h"
#include "DAQEvent.h"
#include "DAQSData.h"

#define NEVENT 5000

DAQEvent * event;
TFile* f, *g;

Float_t  y[NEVENT];

void init(char * output){
	g = new TFile ( output, "RECREATE");
  
}

int main(int argc, char**argv){
#ifdef DAQSOCKET
	if ( argc < 3 ) {
		cout << "Synapsis: LoopDAQEvent <in.root> <out.root> " <<  endl;
		exit(1);
	}
	init ( argv[2] );
	f= new TFile(argv[1]);

	//   Read Tree named "T" in memory. Tree pointer is assigned the same name
	TTree *T = (TTree*)f->Get("T");

	DAQSData *mydaq;
	DAQPayload *mypayload;
	TBranch *branch  = T->GetBranch("event");
	branch->SetAddress(&event);
	Int_t nevent = T->GetEntries();
	Int_t nselected = 0;
	Double_t count = 0;
	for (Int_t i=1;i<nevent;i++) {

		if(i%50 == 0) printf("DAQEvent:%d\n",i);

		T->GetEntry(i);       
		 
		if (event->GetNDAQSDatas() > 687)continue;

		TClonesArray *myarray = event->GetDAQSDatas();
		if (myarray) mydaq = (DAQSData *) myarray->First();
		if (!mydaq) continue;
		cout << "Pointer is: " << myarray << endl;

		cout << "Start printData" << endl;
		mydaq->printData();
		cout << "End printData" << endl;
		mypayload = mydaq->getPayload();
		cout << "Message: " << mypayload->message_ << endl; 
		cout << "Counter: " << mypayload->counter_ << endl; 

		nselected++;
		count += (Double_t) mypayload->counter_/ (nevent - nselected);

		event->Clear();     //clear tracks array
	}
	
	cout << "Count value: " << count << endl;

	f->Close();
	g->cd();
	g->Write();
#else
	cout << "Error: this executable has not been compiled with the DAQSOCKET symbol." <<  endl;
	cout << "Recopile with: make socket" <<  endl;	
	exit(1);
#endif  
}
